/*  trigram
 *  Copyright (C) 1999 YOSHIDA Daisuke.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "gladesig.h"
#include "gladesrc.h"

/* XPM */
static char *ICONTRI[] = {
/* columns rows colors chars-per-pixel */
"32 32 6 1",
"  c Gray0",
". c Red",
"X c #808000",
"o c #808080",
"O c Gray100",
"+ c None",
/* pixels */
"++++++++++++++++++++++++++++++++",
"++++++++++++++++++++++++++++++++",
"++++++++++++++++++++++++++++++++",
"+++++++++++++++++.++++++++++++++",
"++++++++++++++++.....++....+++++",
"+++++++++++...++...........+++++",
"+++++++++..................+++++",
"+++++++++..................+++++",
"+++++++++.................++++++",
"+++++++++......OOOOOO.....++++++",
"++++++++++...OOOOOOOOOO...++++++",
"++++++++++..OOOOOOOOOOOO.+++++++",
"++++++++++.OOOOOOOOOOOO ++++++++",
"+++++++++ OOOOOOOOOOOOO ++++++++",
"+++++++++ OOOOOOOOOOOOOO +++++++",
"++++++++ OOOOOOOOOOOOOOO +++++++",
"++++++++ OOO  OOOOOOOOOOO ++++++",
"+++++++XOOO    OOOOOOOOOO ++++++",
"+++++++XOOO    OOOOOOOOOO ++++++",
"+++++XXOOOOO  OOOOOOOOOOO ++++++",
"++XXXXXOOOOOOOOOOOOOOOOOO ++++++",
"+++XXXXOOOOOOOOOOOOOOOOOO ++++++",
"+++++XXXOoOOOOOOOOOOOOOOO ++++++",
"++++++ OOoOOOOOOOOOOOOOOOO +++++",
"++++++  o OOOOOOOOOOOOOOOO +++++",
"++++++++ OOOOOOOOOOOOOOOOO +++++",
"+++++++ OOOOOOOOOOOOOOOOOO +++++",
"+++++++ OOOOOOOOOOOOOOOOOOO ++++",
"++++++ OOOOOOOOOOOOOOOOOOOO ++++",
"++++++ OOOOOOOOOOOOOOOOOOOOO +++",
"+++++ OOOOOOOOOOOOOOOOOOOOOO +++",
"++++                         +++"
};



GtkWidget*
get_widget                             (GtkWidget       *widget,
                                        gchar           *widget_name)
{
  GtkWidget *parent, *found_widget;

  for (;;)
    {
      if (GTK_IS_MENU (widget))
        parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
      else
        parent = widget->parent;
      if (parent == NULL)
        break;
      widget = parent;
    }

  found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget),
                                                   widget_name);
  if (!found_widget)
    g_warning ("Widget not found: %s", widget_name);
  return found_widget;
}

/* This is an internally used function to set notebook tab widgets. */
void
set_notebook_tab                       (GtkWidget       *notebook,
                                        gint             page_num,
                                        GtkWidget       *widget)
{
  GtkNotebookPage *page;
  GtkWidget *notebook_page;

  page = (GtkNotebookPage*) g_list_nth (GTK_NOTEBOOK (notebook)->children, page_num)->data;
  notebook_page = page->child;
  gtk_widget_ref (notebook_page);
  gtk_notebook_remove_page (GTK_NOTEBOOK (notebook), page_num);
  gtk_notebook_insert_page (GTK_NOTEBOOK (notebook), notebook_page,
                            widget, page_num);
  gtk_widget_unref (notebook_page);
}

static GList *pixmaps_directories = NULL;

/* Use this function to set the directory containing installed pixmaps. */
void
add_pixmap_directory                   (gchar           *directory)
{
  pixmaps_directories = g_list_prepend (pixmaps_directories, g_strdup (directory));
}

/* This is an internally used function to check if a pixmap file exists. */
#ifndef G_DIR_SEPARATOR_S
#define G_DIR_SEPARATOR_S "/"
#endif
gchar*
check_file_exists                      (gchar           *directory,
                                        gchar           *filename)
{
  gchar *full_filename;
  struct stat s;
  gint status;

  full_filename = g_malloc (strlen (directory) + 1 + strlen (filename) + 1);
  strcpy (full_filename, directory);
  strcat (full_filename, G_DIR_SEPARATOR_S);
  strcat (full_filename, filename);

  status = stat (full_filename, &s);
  if (status == 0 && S_ISREG (s.st_mode))
    return full_filename;
  g_free (full_filename);
  return NULL;
}

/* This is an internally used function to create pixmaps. */
GtkWidget*
create_pixmap                          (GtkWidget       *widget,
                                        gchar           *filename)
{
  gchar *found_filename = NULL;
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;
  GList *elem;

  /* We first try any pixmaps directories set by the application. */
  elem = pixmaps_directories;
  while (elem)
    {
      found_filename = check_file_exists ((gchar*)elem->data, filename);
      if (found_filename)
        break;
      elem = elem->next;
    }

  /* If we haven't found the pixmap, try the source directory. */
  if (!found_filename)
    {
      found_filename = check_file_exists ("pixmaps", filename);
    }

  if (!found_filename)
    {
      g_print ("Couldn't find pixmap file: %s", filename);
      return NULL;
    }

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm (NULL, colormap, &mask,
                                                   NULL, found_filename);
  g_free (found_filename);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_pixmap_d                        (GtkWidget       *widget,
                                        gchar           *data[])
{
  GdkColormap *colormap;
  GdkPixmap *gdkpixmap;
  GdkBitmap *mask;
  GtkWidget *pixmap;

  colormap = gtk_widget_get_colormap (widget);
  gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d (NULL, colormap, &mask,
						     NULL, data);
  if (gdkpixmap == NULL)
    return NULL;
  pixmap = gtk_pixmap_new (gdkpixmap, mask);
  gdk_pixmap_unref (gdkpixmap);
  gdk_bitmap_unref (mask);
  return pixmap;
}

GtkWidget*
create_window1 ()
{
  GtkWidget *window1;
  GtkWidget *alignment1;
  GtkWidget *vbox1;
  GtkWidget *menubar1;
  GtkWidget *File;
  GtkWidget *File_menu;
  GtkWidget *New;
  GtkWidget *Open;
  GtkWidget *Save;
  GtkWidget *Save_as;
  GtkWidget *__________1;
  GtkWidget *Exit;
  GtkWidget *Graph;
  GtkWidget *Graph_menu;
  GtkWidget *prop;
  GtkWidget *__________3;
  GtkWidget *Output_main_plot;
  GtkWidget *Output_plotted_area;
  GtkWidget *__________4;
  GtkWidget *Print_graph;
  GtkWidget *Help;
  GtkWidget *Help_menu;
  GtkWidget *About;
  GtkWidget *clist1;
  GtkWidget *label1;
  GtkWidget *label2;
  GtkWidget *label3;
  GtkWidget *label4;
  GtkWidget *label5;
  GtkWidget *label6;
  GtkWidget *hbox1;
  GtkWidget *vbox2;
  GtkWidget *hbox2;
  GtkWidget *vbox3;
  GtkWidget *label13;
  GtkWidget *entry2;
  GtkWidget *vbox4;
  GtkWidget *label14;
  GtkWidget *entry3;
  GtkWidget *hbox3;
  GtkWidget *vbox5;
  GtkWidget *label15;
  GtkWidget *entry4;
  GtkWidget *vbox6;
  GtkWidget *label16;
  GtkWidget *entry5;
  GtkWidget *vbox7;
  GtkWidget *label17;
  GtkWidget *entry6;
  GtkWidget *vbox9;
  GtkWidget *label19;
  GtkWidget *entry8;
  GtkWidget *button3;
  GtkWidget *vbox11;
  GtkWidget *pixmap4;
  GtkWidget *label22;
  GtkWidget *statusbar1;
#ifdef GTK_HAVE_FEATURES_1_1_4
  GtkWidget *scrwindow1;
#endif

  /* CList򤷤ȤΥХåؿѥǡ */
  static GtkWidget *inputentry[] = {NULL, NULL, NULL, NULL, NULL, NULL};

  window1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_object_set_data (GTK_OBJECT (window1), "window1", window1);
  gtk_widget_set_usize (window1, 570, 450);
  gtk_signal_connect (GTK_OBJECT (window1), "delete_event",
                      GTK_SIGNAL_FUNC (on_window1_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (window1), "Trigram");

  /*
   * Хåؿ˥åȤϤ
   * ˥åȤä֤ɬפ
   */
  clist1 = gtk_clist_new (6);
  entry2 = gtk_entry_new ();
  entry3 = gtk_entry_new ();
  entry4 = gtk_entry_new ();
  entry5 = gtk_entry_new ();
  entry6 = gtk_entry_new ();
  entry8 = gtk_entry_new ();
  inputentry [0] = entry2;
  inputentry [1] = entry3;
  inputentry [2] = entry4;
  inputentry [3] = entry5;
  inputentry [4] = entry6;
  inputentry [5] = entry8;
  /*
   * ޤ
   */

  alignment1 = gtk_alignment_new (0.5, 0.5, 1, 1);
  gtk_object_set_data (GTK_OBJECT (window1), "alignment1", alignment1);
  gtk_widget_show (alignment1);
  gtk_container_add (GTK_CONTAINER (window1), alignment1);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox1", vbox1);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (alignment1), vbox1);

  menubar1 = gtk_menu_bar_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "menubar1", menubar1);
  gtk_widget_show (menubar1);
  gtk_box_pack_start (GTK_BOX (vbox1), menubar1, FALSE, TRUE, 0);

  File = gtk_menu_item_new_with_label ("File");
  gtk_object_set_data (GTK_OBJECT (window1), "File", File);
  gtk_widget_show (File);
  gtk_container_add (GTK_CONTAINER (menubar1), File);

  File_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "File_menu", File_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (File), File_menu);

  New = gtk_menu_item_new_with_label ("New...");
  gtk_object_set_data (GTK_OBJECT (window1), "New", New);
  gtk_widget_show (New);
  gtk_container_add (GTK_CONTAINER (File_menu), New);
  gtk_signal_connect (GTK_OBJECT (New), "activate",
                      GTK_SIGNAL_FUNC (on_New_activate),
                      clist1);

  Open = gtk_menu_item_new_with_label ("Open...");
  gtk_object_set_data (GTK_OBJECT (window1), "Open", Open);
  gtk_widget_show (Open);
  gtk_container_add (GTK_CONTAINER (File_menu), Open);
  gtk_signal_connect (GTK_OBJECT (Open), "activate",
                      GTK_SIGNAL_FUNC (on_Open_activate),
                      clist1);

  Save = gtk_menu_item_new_with_label ("Save");
  gtk_object_set_data (GTK_OBJECT (window1), "Save", Save);
  gtk_widget_show (Save);
  gtk_container_add (GTK_CONTAINER (File_menu), Save);
  gtk_signal_connect (GTK_OBJECT (Save), "activate",
                      GTK_SIGNAL_FUNC (on_Save_activate),
                      clist1);

  Save_as = gtk_menu_item_new_with_label ("Save as...");
  gtk_object_set_data (GTK_OBJECT (window1), "Save_as", Save_as);
  gtk_widget_show (Save_as);
  gtk_container_add (GTK_CONTAINER (File_menu), Save_as);
  gtk_signal_connect (GTK_OBJECT (Save_as), "activate",
                      GTK_SIGNAL_FUNC (on_Save_as_activate),
                      clist1);

  __________1 = gtk_menu_item_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "__________1", __________1);
  gtk_widget_show (__________1);
  gtk_container_add (GTK_CONTAINER (File_menu), __________1);

  Exit = gtk_menu_item_new_with_label ("Exit");
  gtk_object_set_data (GTK_OBJECT (window1), "Exit", Exit);
  gtk_widget_show (Exit);
  gtk_container_add (GTK_CONTAINER (File_menu), Exit);
  gtk_signal_connect (GTK_OBJECT (Exit), "activate",
                      GTK_SIGNAL_FUNC (on_Exit_activate),
                      NULL);

  Graph = gtk_menu_item_new_with_label ("Graph");
  gtk_object_set_data (GTK_OBJECT (window1), "Graph", Graph);
  gtk_widget_show (Graph);
  gtk_container_add (GTK_CONTAINER (menubar1), Graph);

  Graph_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "Graph_menu", Graph_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (Graph), Graph_menu);

  prop = gtk_menu_item_new_with_label ("Property...");
  gtk_object_set_data (GTK_OBJECT (window1), "prop", prop);
  gtk_widget_show (prop);
  gtk_container_add (GTK_CONTAINER (Graph_menu), prop);
  gtk_signal_connect (GTK_OBJECT (prop), "activate",
                      GTK_SIGNAL_FUNC (on_prop_activate),
                      NULL);

  __________3 = gtk_menu_item_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "__________3", __________3);
  gtk_widget_show (__________3);
  gtk_container_add (GTK_CONTAINER (Graph_menu), __________3);

  Output_main_plot = gtk_menu_item_new_with_label
    ("Output EPS (whole)...");
  gtk_object_set_data
    (GTK_OBJECT (window1), "Output_main_plot", Output_main_plot);
  gtk_widget_show (Output_main_plot);
  gtk_container_add (GTK_CONTAINER (Graph_menu), Output_main_plot);
  gtk_signal_connect (GTK_OBJECT (Output_main_plot), "activate",
                      GTK_SIGNAL_FUNC (on_Output_main_plot_activate),
                      clist1);

  Output_plotted_area = gtk_menu_item_new_with_label
    ("Output EPS File (part)...");
  gtk_object_set_data
    (GTK_OBJECT (window1), "Output_plotted_area", Output_plotted_area);
  gtk_widget_show (Output_plotted_area);
  gtk_container_add (GTK_CONTAINER (Graph_menu), Output_plotted_area);
  gtk_signal_connect (GTK_OBJECT (Output_plotted_area), "activate",
                      GTK_SIGNAL_FUNC (on_Output_plotted_area_activate),
                      clist1);

  __________4 = gtk_menu_item_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "__________4", __________4);
  gtk_widget_show (__________4);
  gtk_container_add (GTK_CONTAINER (Graph_menu), __________4);

  Print_graph = gtk_menu_item_new_with_label ("Print...");
  gtk_object_set_data (GTK_OBJECT (window1), "Print_graph", Print_graph);
  gtk_widget_show (Print_graph);
  gtk_container_add (GTK_CONTAINER (Graph_menu), Print_graph);
  gtk_signal_connect (GTK_OBJECT (Print_graph), "activate",
                      GTK_SIGNAL_FUNC (on_Print_graph_activate),
                      clist1);

  Help = gtk_menu_item_new_with_label ("Help");
  gtk_object_set_data (GTK_OBJECT (window1), "Help", Help);
  gtk_widget_show (Help);
  gtk_container_add (GTK_CONTAINER (menubar1), Help);

  Help_menu = gtk_menu_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "Help_menu", Help_menu);
  gtk_menu_item_set_submenu (GTK_MENU_ITEM (Help), Help_menu);

  About = gtk_menu_item_new_with_label ("About...");
  gtk_object_set_data (GTK_OBJECT (window1), "About", About);
  gtk_widget_show (About);
  gtk_container_add (GTK_CONTAINER (Help_menu), About);
  gtk_signal_connect (GTK_OBJECT (About), "activate",
                      GTK_SIGNAL_FUNC (on_About_activate),
                      NULL);

  /*
   * ιԤƬ˻äƹԤޤ
   clist1 = gtk_clist_new (6);
   */
  gtk_object_set_data (GTK_OBJECT (window1), "clist1", clist1);
  gtk_widget_show (clist1);
  gtk_signal_connect (GTK_OBJECT (clist1), "select_row",
                      GTK_SIGNAL_FUNC (on_clist1_select_row),
                      inputentry);
  gtk_signal_connect (GTK_OBJECT (clist1), "unselect_row",
                      GTK_SIGNAL_FUNC (on_clist1_unselect_row),
                      inputentry);
  gtk_signal_connect (GTK_OBJECT (clist1), "click_column",
                      GTK_SIGNAL_FUNC (on_clist1_click_column),
                      NULL);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 0, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 1, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 2, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 3, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 4, 80);
  gtk_clist_set_column_width (GTK_CLIST (clist1), 5, 80);
  gtk_clist_column_titles_show (GTK_CLIST (clist1));

#ifdef GTK_HAVE_FEATURES_1_1_4
  scrwindow1 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrwindow1);
  gtk_scrolled_window_set_policy 
    (GTK_SCROLLED_WINDOW(scrwindow1), GTK_POLICY_AUTOMATIC,
     GTK_POLICY_AUTOMATIC);
  gtk_container_add (GTK_CONTAINER(scrwindow1), clist1);
  gtk_box_pack_start (GTK_BOX (vbox1), scrwindow1, TRUE, TRUE, 0);
#else
  gtk_clist_set_policy
    (GTK_CLIST (clist1), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox1), clist1, TRUE, TRUE, 0);
#endif
  on_New_activate(NULL, clist1);

  label1 = gtk_label_new ("Name");
  gtk_object_set_data (GTK_OBJECT (window1), "label1", label1);
  gtk_widget_show (label1);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 0, label1);

  label2 = gtk_label_new ("Type");
  gtk_object_set_data (GTK_OBJECT (window1), "label2", label2);
  gtk_widget_show (label2);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 1, label2);

  label3 = gtk_label_new ("X");
  gtk_object_set_data (GTK_OBJECT (window1), "label3", label3);
  gtk_widget_show (label3);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 2, label3);
  gtk_widget_set_usize (label3, 9, -2);

  label4 = gtk_label_new ("Y");
  gtk_object_set_data (GTK_OBJECT (window1), "label4", label4);
  gtk_widget_show (label4);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 3, label4);
  gtk_widget_set_usize (label4, 9, -2);

  label5 = gtk_label_new ("Z");
  gtk_object_set_data (GTK_OBJECT (window1), "label5", label5);
  gtk_widget_show (label5);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 4, label5);
  gtk_widget_set_usize (label5, 9, -2);

  label6 = gtk_label_new ("Memo");
  gtk_object_set_data (GTK_OBJECT (window1), "label6", label6);
  gtk_widget_show (label6);
  gtk_clist_set_column_widget (GTK_CLIST (clist1), 5, label6);
  gtk_widget_set_usize (label6, 34, -2);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "hbox1", hbox1);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox2", vbox2);
  gtk_widget_show (vbox2);
  gtk_box_pack_start (GTK_BOX (hbox1), vbox2, FALSE, FALSE, 0);

  hbox2 = gtk_hbox_new (TRUE, 5);
  gtk_object_set_data (GTK_OBJECT (window1), "hbox2", hbox2);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbox2), 5);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox3", vbox3);
  gtk_widget_show (vbox3);
  gtk_box_pack_start (GTK_BOX (hbox2), vbox3, FALSE, FALSE, 0);

  label13 = gtk_label_new ("Name:");
  gtk_object_set_data (GTK_OBJECT (window1), "label13", label13);
  gtk_widget_show (label13);
  gtk_box_pack_start (GTK_BOX (vbox3), label13, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label13), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label13), 0.0600002, 0.5);

  /*
   * Ƭ˰ư
   entry2 = gtk_entry_new ();
   */
  gtk_object_set_data (GTK_OBJECT (window1), "entry2", entry2);
  gtk_widget_show (entry2);
  gtk_box_pack_start (GTK_BOX (vbox3), entry2, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry2), "changed",
                      GTK_SIGNAL_FUNC (on_entry2_changed),
                      clist1);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox4", vbox4);
  gtk_widget_show (vbox4);
  gtk_box_pack_start (GTK_BOX (hbox2), vbox4, FALSE, FALSE, 0);

  label14 = gtk_label_new ("Type:");
  gtk_object_set_data (GTK_OBJECT (window1), "label14", label14);
  gtk_widget_show (label14);
  gtk_box_pack_start (GTK_BOX (vbox4), label14, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label14), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label14), 0.0600002, 0.5);

  /*
   * Ƭ˰ư
   entry3 = gtk_entry_new ();
   */
  gtk_object_set_data (GTK_OBJECT (window1), "entry3", entry3);
  gtk_widget_show (entry3);
  gtk_box_pack_start (GTK_BOX (vbox4), entry3, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry3), "changed",
                      GTK_SIGNAL_FUNC (on_entry3_changed),
                      clist1);

  hbox3 = gtk_hbox_new (TRUE, 5);
  gtk_object_set_data (GTK_OBJECT (window1), "hbox3", hbox3);
  gtk_widget_show (hbox3);
  gtk_box_pack_start (GTK_BOX (vbox2), hbox3, FALSE, FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (hbox3), 5);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox5", vbox5);
  gtk_widget_show (vbox5);
  gtk_box_pack_start (GTK_BOX (hbox3), vbox5, FALSE, FALSE, 0);

  label15 = gtk_label_new ("X:");
  gtk_object_set_data (GTK_OBJECT (window1), "label15", label15);
  gtk_widget_show (label15);
  gtk_box_pack_start (GTK_BOX (vbox5), label15, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label15), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label15), 0.0600002, 0.5);

  /*
   * Ƭ˰ư
   entry4 = gtk_entry_new ();
   */
  gtk_object_set_data (GTK_OBJECT (window1), "entry4", entry4);
  gtk_widget_show (entry4);
  gtk_box_pack_start (GTK_BOX (vbox5), entry4, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry4), "changed",
                      GTK_SIGNAL_FUNC (on_entry4_changed),
                      clist1);

  vbox6 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox6", vbox6);
  gtk_widget_show (vbox6);
  gtk_box_pack_start (GTK_BOX (hbox3), vbox6, FALSE, FALSE, 0);

  label16 = gtk_label_new ("Y:");
  gtk_object_set_data (GTK_OBJECT (window1), "label16", label16);
  gtk_widget_show (label16);
  gtk_box_pack_start (GTK_BOX (vbox6), label16, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label16), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label16), 0.0600002, 0.5);

  /*
   * Ƭ˰ư
   entry5 = gtk_entry_new ();
   */
  gtk_object_set_data (GTK_OBJECT (window1), "entry5", entry5);
  gtk_widget_show (entry5);
  gtk_box_pack_start (GTK_BOX (vbox6), entry5, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry5), "changed",
                      GTK_SIGNAL_FUNC (on_entry5_changed),
                      clist1);

  vbox7 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox7", vbox7);
  gtk_widget_show (vbox7);
  gtk_box_pack_start (GTK_BOX (hbox3), vbox7, FALSE, FALSE, 0);

  label17 = gtk_label_new ("Z:");
  gtk_object_set_data (GTK_OBJECT (window1), "label17", label17);
  gtk_widget_show (label17);
  gtk_box_pack_start (GTK_BOX (vbox7), label17, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label17), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label17), 0.0600002, 0.5);

  /*
   * Ƭ˰ư
   entry6 = gtk_entry_new ();
   */
  gtk_object_set_data (GTK_OBJECT (window1), "entry6", entry6);
  gtk_widget_show (entry6);
  gtk_box_pack_start (GTK_BOX (vbox7), entry6, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry6), "changed",
                      GTK_SIGNAL_FUNC (on_entry6_changed),
                      clist1);

  vbox9 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox9", vbox9);
  gtk_widget_show (vbox9);
  gtk_box_pack_start (GTK_BOX (vbox2), vbox9, FALSE, FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (vbox9), 6);

  label19 = gtk_label_new ("Memo:");
  gtk_object_set_data (GTK_OBJECT (window1), "label19", label19);
  gtk_widget_show (label19);
  gtk_box_pack_start (GTK_BOX (vbox9), label19, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label19), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label19), 0.0200002, 0.5);

  /*
   * Ƭ˰ư
   entry8 = gtk_entry_new ();
   */
  gtk_object_set_data (GTK_OBJECT (window1), "entry8", entry8);
  gtk_widget_show (entry8);
  gtk_box_pack_start (GTK_BOX (vbox9), entry8, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry8), "changed",
                      GTK_SIGNAL_FUNC (on_entry8_changed),
                      clist1);

  button3 = gtk_button_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "button3", button3);
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (hbox1), button3, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (on_button3_clicked),
                      NULL);

  vbox11 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window1), "vbox11", vbox11);
  gtk_widget_show (vbox11);
  gtk_container_add (GTK_CONTAINER (button3), vbox11);

  pixmap4 = create_pixmap_d (window1, ICONTRI);
  if (pixmap4 == NULL)
    g_error ("Couldn't create pixmap");
  gtk_object_set_data (GTK_OBJECT (window1), "pixmap4", pixmap4);
  gtk_widget_show (pixmap4);
  gtk_box_pack_start (GTK_BOX (vbox11), pixmap4, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (pixmap4), 0.5, 0.9);

  label22 = gtk_label_new ("About\ntrigram");
  gtk_object_set_data (GTK_OBJECT (window1), "label22", label22);
  gtk_widget_show (label22);
  gtk_box_pack_start (GTK_BOX (vbox11), label22, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (label22), 0.5, 0.01);

  statusbar1 = gtk_statusbar_new ();
  gtk_object_set_data (GTK_OBJECT (window1), "statusbar1", statusbar1);
  gtk_widget_show (statusbar1);
  gtk_box_pack_start (GTK_BOX (vbox1), statusbar1, FALSE, TRUE, 0);

  return window1;
}

GtkWidget*
create_window2 ()
{
  GtkWidget *window2;
  GtkWidget *button4;
  GtkWidget *hbox4;
  GtkWidget *pixmap5;
  GtkWidget *label23;

  gchar *tmp_help1 = "Trigram ver %s (gtk)\n\n"
    "Copyright (c) 1999 YOSHIDA Daisuke.\n\n"
    "Trigram is a free software with ABSOLUTELY NO\n"
    "WARRANTY. You are welcome to redistribute it\n"
    "under certain conditions. See the file 'COPYING'\n"
    "for details.\n\n"
    "mailto:quippa@geocities.co.jp\n";
  gchar *tmp_help2;
  tmp_help2 = g_malloc (strlen(tmp_help1) + 20);
  sprintf (tmp_help2, tmp_help1, VERSION);

  window2 = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (window2), "window2", window2);
  gtk_container_border_width (GTK_CONTAINER (window2), 8);
  gtk_signal_connect (GTK_OBJECT (window2), "delete_event",
                      GTK_SIGNAL_FUNC (on_window2_delete_event),
                      NULL);
  gtk_window_set_title (GTK_WINDOW (window2), "About trigram");
  gtk_window_position (GTK_WINDOW (window2), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (window2), TRUE, TRUE, FALSE);

  button4 = gtk_button_new ();
  gtk_object_set_data (GTK_OBJECT (window2), "button4", button4);
  gtk_widget_show (button4);
  gtk_container_add (GTK_CONTAINER (window2), button4);
  gtk_signal_connect (GTK_OBJECT (button4), "clicked",
                      GTK_SIGNAL_FUNC (on_button4_clicked),
                      NULL);

  hbox4 = gtk_hbox_new (FALSE, 20);
  gtk_object_set_data (GTK_OBJECT (window2), "hbox4", hbox4);
  gtk_widget_show (hbox4);
  gtk_container_add (GTK_CONTAINER (button4), hbox4);
  gtk_container_border_width (GTK_CONTAINER (hbox4), 5);

  pixmap5 = create_pixmap_d (window2, ICONTRI);
  if (pixmap5 == NULL)
    g_error ("Couldn't create pixmap");
  gtk_object_set_data (GTK_OBJECT (window2), "pixmap5", pixmap5);
  gtk_widget_show (pixmap5);
  gtk_box_pack_start (GTK_BOX (hbox4), pixmap5, TRUE, TRUE, 0);
  gtk_misc_set_alignment (GTK_MISC (pixmap5), 0.8, 0.25);

  label23 = gtk_label_new (tmp_help2);
  gtk_object_set_data (GTK_OBJECT (window2), "label23", label23);
  gtk_label_set_justify (GTK_LABEL(label23), GTK_JUSTIFY_LEFT);
  gtk_widget_show (label23);
  gtk_box_pack_start (GTK_BOX (hbox4), label23, TRUE, TRUE, 0);
  g_free (tmp_help2);

  return window2;
}

GtkWidget*
create_fileselection1 ()
{
  GtkWidget *fileselection1;
  GtkWidget *ok_button2;
  GtkWidget *cancel_button2;

  fileselection1 = gtk_file_selection_new ("\245\325\245\241\245\244\245\353\301\252\302\362");
  gtk_object_set_data (GTK_OBJECT (fileselection1), "fileselection1", fileselection1);
  gtk_container_border_width (GTK_CONTAINER (fileselection1), 10);
  gtk_window_position (GTK_WINDOW (fileselection1), GTK_WIN_POS_CENTER);

  ok_button2 = GTK_FILE_SELECTION (fileselection1)->ok_button;
  gtk_object_set_data (GTK_OBJECT (fileselection1), "ok_button2", ok_button2);
  gtk_widget_show (ok_button2);
  GTK_WIDGET_SET_FLAGS (ok_button2, GTK_CAN_DEFAULT);

  cancel_button2 = GTK_FILE_SELECTION (fileselection1)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (fileselection1), "cancel_button2", cancel_button2);
  gtk_widget_show (cancel_button2);
  GTK_WIDGET_SET_FLAGS (cancel_button2, GTK_CAN_DEFAULT);

  /* gladeꤹ٤ΤʤäΤǼư */
  gtk_signal_connect (GTK_OBJECT(ok_button2), "clicked",
		      GTK_SIGNAL_FUNC(on_ok_button1_clicked), fileselection1);
  gtk_signal_connect (GTK_OBJECT(cancel_button2), "clicked",
		      GTK_SIGNAL_FUNC(on_cancel_button1_clicked), fileselection1);

  return fileselection1;
}

GtkWidget*
create_fileselection2 ()
{
  GtkWidget *fileselection2;
  GtkWidget *ok_button3;
  GtkWidget *cancel_button3;

  fileselection2 = gtk_file_selection_new ("\245\325\245\241\245\244\245\353\301\252\302\362");
  gtk_object_set_data (GTK_OBJECT (fileselection2), "fileselection2", fileselection2);
  gtk_container_border_width (GTK_CONTAINER (fileselection2), 10);
  gtk_window_position (GTK_WINDOW (fileselection2), GTK_WIN_POS_CENTER);

  ok_button3 = GTK_FILE_SELECTION (fileselection2)->ok_button;
  gtk_object_set_data (GTK_OBJECT (fileselection2), "ok_button3", ok_button3);
  gtk_widget_show (ok_button3);
  GTK_WIDGET_SET_FLAGS (ok_button3, GTK_CAN_DEFAULT);

  cancel_button3 = GTK_FILE_SELECTION (fileselection2)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (fileselection2), "cancel_button3", cancel_button3);
  gtk_widget_show (cancel_button3);
  GTK_WIDGET_SET_FLAGS (cancel_button3, GTK_CAN_DEFAULT);

  /* gladeꤹ٤ΤʤäΤǼư */
  gtk_signal_connect (GTK_OBJECT(ok_button3), "clicked",
		      GTK_SIGNAL_FUNC(on_ok_button2_clicked), fileselection2);
  gtk_signal_connect (GTK_OBJECT(cancel_button3), "clicked",
		      GTK_SIGNAL_FUNC(on_cancel_button2_clicked), fileselection2);

  return fileselection2;
}

GtkWidget*
create_fileselection3 ()
{
  GtkWidget *fileselection3;
  GtkWidget *ok_button4;
  GtkWidget *cancel_button4;

  fileselection3 = gtk_file_selection_new ("\245\325\245\241\245\244\245\353\301\252\302\362");
  gtk_object_set_data (GTK_OBJECT (fileselection3), "fileselection3", fileselection3);
  gtk_container_border_width (GTK_CONTAINER (fileselection3), 10);
  gtk_window_position (GTK_WINDOW (fileselection3), GTK_WIN_POS_CENTER);

  ok_button4 = GTK_FILE_SELECTION (fileselection3)->ok_button;
  gtk_object_set_data (GTK_OBJECT (fileselection3), "ok_button4", ok_button4);
  gtk_widget_show (ok_button4);
  GTK_WIDGET_SET_FLAGS (ok_button4, GTK_CAN_DEFAULT);

  cancel_button4 = GTK_FILE_SELECTION (fileselection3)->cancel_button;
  gtk_object_set_data (GTK_OBJECT (fileselection3), "cancel_button4", cancel_button4);
  gtk_widget_show (cancel_button4);
  GTK_WIDGET_SET_FLAGS (cancel_button4, GTK_CAN_DEFAULT);

  /* gladeꤹ٤ΤʤäΤǼư */
  gtk_signal_connect (GTK_OBJECT(ok_button4), "clicked",
		      GTK_SIGNAL_FUNC(on_ok_button3_clicked), fileselection3);
  gtk_signal_connect (GTK_OBJECT(cancel_button4), "clicked",
		      GTK_SIGNAL_FUNC(on_cancel_button3_clicked), fileselection3);

  return fileselection3;
}

GtkWidget*
create_window3 ()
{
  GtkWidget *window3;
  GtkWidget *vbox12;
  GtkWidget *frame3;
  GtkWidget *hbox9;
  GtkWidget *hbox10;
  GtkWidget *label27;
  GtkWidget *entry12;
  GtkWidget *hbox11;
  GtkWidget *label28;
  GtkWidget *entry13;
  GtkWidget *hbox12;
  GtkWidget *label29;
  GtkWidget *entry14;
  GtkWidget *frame1;
  GtkWidget *table2;
  GSList *table2_group = NULL;
  GtkWidget *radiobutton1;
  GtkWidget *radiobutton2;
  GtkWidget *radiobutton3;
  GtkWidget *radiobutton4;
  GtkWidget *radiobutton5;
  GtkWidget *radiobutton6;
  GtkWidget *radiobutton7;
  GtkWidget *radiobutton8;
  GtkWidget *frame2;
  GtkWidget *table3;
  GSList *table3_group = NULL;
  GtkWidget *radiobutton9;
  GtkWidget *radiobutton10;
  GtkWidget *radiobutton11;
  GtkWidget *radiobutton12;
  GtkWidget *radiobutton13;
  GtkWidget *radiobutton14;
  GtkWidget *radiobutton15;
  GtkWidget *radiobutton16;
  GtkWidget *hbox13;
  GtkWidget *button5;

  /* ХåؿϤΥå */
  static GtkWidget *propdata[] = {NULL, NULL, NULL, NULL, NULL,
				  NULL, NULL, NULL, NULL, NULL,
				  NULL, NULL, NULL, NULL, NULL,
				  NULL, NULL, NULL, NULL};


  window3 = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_object_set_data (GTK_OBJECT (window3), "window3", window3);
  gtk_window_set_title (GTK_WINDOW (window3), "Graph property");
  gtk_window_position (GTK_WINDOW (window3), GTK_WIN_POS_CENTER);

  vbox12 = gtk_vbox_new (FALSE, 10);
  gtk_object_set_data (GTK_OBJECT (window3), "vbox12", vbox12);
  gtk_widget_show (vbox12);
  gtk_container_add (GTK_CONTAINER (window3), vbox12);
  gtk_container_border_width (GTK_CONTAINER (vbox12), 10);

  frame3 = gtk_frame_new ("Axes label");
  gtk_object_set_data (GTK_OBJECT (window3), "frame3", frame3);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox12), frame3, TRUE, TRUE, 0);

  hbox9 = gtk_hbox_new (TRUE, 10);
  gtk_object_set_data (GTK_OBJECT (window3), "hbox9", hbox9);
  gtk_widget_show (hbox9);
  gtk_container_add (GTK_CONTAINER (frame3), hbox9);
  gtk_container_border_width (GTK_CONTAINER (hbox9), 5);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window3), "hbox10", hbox10);
  gtk_widget_show (hbox10);
  gtk_box_pack_start (GTK_BOX (hbox9), hbox10, TRUE, TRUE, 0);

  label27 = gtk_label_new ("X:");
  gtk_object_set_data (GTK_OBJECT (window3), "label27", label27);
  gtk_widget_show (label27);
  gtk_box_pack_start (GTK_BOX (hbox10), label27, TRUE, TRUE, 0);

  entry12 = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (window3), "entry12", entry12);
  gtk_widget_show (entry12);
  gtk_box_pack_start (GTK_BOX (hbox10), entry12, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry12), "changed",
                      GTK_SIGNAL_FUNC (on_entry12_changed),
                      NULL);

  hbox11 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window3), "hbox11", hbox11);
  gtk_widget_show (hbox11);
  gtk_box_pack_start (GTK_BOX (hbox9), hbox11, TRUE, TRUE, 0);

  label28 = gtk_label_new ("Y:");
  gtk_object_set_data (GTK_OBJECT (window3), "label28", label28);
  gtk_widget_show (label28);
  gtk_box_pack_start (GTK_BOX (hbox11), label28, TRUE, TRUE, 0);

  entry13 = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (window3), "entry13", entry13);
  gtk_widget_show (entry13);
  gtk_box_pack_start (GTK_BOX (hbox11), entry13, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry13), "changed",
                      GTK_SIGNAL_FUNC (on_entry13_changed),
                      NULL);

  hbox12 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window3), "hbox12", hbox12);
  gtk_widget_show (hbox12);
  gtk_box_pack_start (GTK_BOX (hbox9), hbox12, TRUE, TRUE, 0);

  label29 = gtk_label_new ("Z:");
  gtk_object_set_data (GTK_OBJECT (window3), "label29", label29);
  gtk_widget_show (label29);
  gtk_box_pack_start (GTK_BOX (hbox12), label29, TRUE, TRUE, 0);

  entry14 = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (window3), "entry14", entry14);
  gtk_widget_show (entry14);
  gtk_box_pack_start (GTK_BOX (hbox12), entry14, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (entry14), "changed",
                      GTK_SIGNAL_FUNC (on_entry14_changed),
                      NULL);

  frame1 = gtk_frame_new ("Line interval");
  gtk_object_set_data (GTK_OBJECT (window3), "frame1", frame1);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox12), frame1, TRUE, TRUE, 0);

  table2 = gtk_table_new (4, 2, TRUE);
  gtk_object_set_data (GTK_OBJECT (window3), "table2", table2);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (frame1), table2);
  gtk_container_border_width (GTK_CONTAINER (table2), 5);

  radiobutton1 = gtk_radio_button_new_with_label (table2_group, "No line");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton1", radiobutton1);
  gtk_widget_show (radiobutton1);
  gtk_table_attach (GTK_TABLE (table2), radiobutton1, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton1_toggled),
                      NULL);

  radiobutton2 = gtk_radio_button_new_with_label (table2_group, "every 1%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton2", radiobutton2);
  gtk_widget_show (radiobutton2);
  gtk_table_attach (GTK_TABLE (table2), radiobutton2, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton2), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton2_toggled),
                      NULL);

  radiobutton3 = gtk_radio_button_new_with_label (table2_group, "every 2%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton3));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton3", radiobutton3);
  gtk_widget_show (radiobutton3);
  gtk_table_attach (GTK_TABLE (table2), radiobutton3, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton3), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton3_toggled),
                      NULL);

  radiobutton4 = gtk_radio_button_new_with_label (table2_group, "every 5%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton4));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton4", radiobutton4);
  gtk_widget_show (radiobutton4);
  gtk_table_attach (GTK_TABLE (table2), radiobutton4, 0, 1, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton4), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton4_toggled),
                      NULL);

  radiobutton5 = gtk_radio_button_new_with_label (table2_group, "every 10%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton5));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton5", radiobutton5);
  gtk_widget_show (radiobutton5);
  gtk_table_attach (GTK_TABLE (table2), radiobutton5, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton5), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton5_toggled),
                      NULL);

  radiobutton6 = gtk_radio_button_new_with_label (table2_group, "every 20%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton6));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton6", radiobutton6);
  gtk_widget_show (radiobutton6);
  gtk_table_attach (GTK_TABLE (table2), radiobutton6, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton6), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton6_toggled),
                      NULL);

  radiobutton7 = gtk_radio_button_new_with_label (table2_group, "every 25%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton7));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton7", radiobutton7);
  gtk_widget_show (radiobutton7);
  gtk_table_attach (GTK_TABLE (table2), radiobutton7, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton7), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton7_toggled),
                      NULL);

  radiobutton8 = gtk_radio_button_new_with_label (table2_group, "every 50%");
  table2_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton8));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton8", radiobutton8);
  gtk_widget_show (radiobutton8);
  gtk_table_attach (GTK_TABLE (table2), radiobutton8, 1, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton8), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton8_toggled),
                      NULL);

  frame2 = gtk_frame_new ("Number interval");
  gtk_object_set_data (GTK_OBJECT (window3), "frame2", frame2);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox12), frame2, TRUE, TRUE, 0);

  table3 = gtk_table_new (4, 2, TRUE);
  gtk_object_set_data (GTK_OBJECT (window3), "table3", table3);
  gtk_widget_show (table3);
  gtk_container_add (GTK_CONTAINER (frame2), table3);
  gtk_container_border_width (GTK_CONTAINER (table3), 5);

  radiobutton9 = gtk_radio_button_new_with_label (table3_group, "No number");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton9));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton9", radiobutton9);
  gtk_widget_show (radiobutton9);
  gtk_table_attach (GTK_TABLE (table3), radiobutton9, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton9), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton9_toggled),
                      NULL);

  radiobutton10 = gtk_radio_button_new_with_label (table3_group, "every 1%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton10));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton10", radiobutton10);
  gtk_widget_show (radiobutton10);
  gtk_table_attach (GTK_TABLE (table3), radiobutton10, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton10), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton10_toggled),
                      NULL);

  radiobutton11 = gtk_radio_button_new_with_label (table3_group, "every 2%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton11));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton11", radiobutton11);
  gtk_widget_show (radiobutton11);
  gtk_table_attach (GTK_TABLE (table3), radiobutton11, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton11), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton11_toggled),
                      NULL);

  radiobutton12 = gtk_radio_button_new_with_label (table3_group, "every 5%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton12));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton12", radiobutton12);
  gtk_widget_show (radiobutton12);
  gtk_table_attach (GTK_TABLE (table3), radiobutton12, 0, 1, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton12), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton12_toggled),
                      NULL);

  radiobutton13 = gtk_radio_button_new_with_label (table3_group, "every 10%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton13));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton13", radiobutton13);
  gtk_widget_show (radiobutton13);
  gtk_table_attach (GTK_TABLE (table3), radiobutton13, 1, 2, 0, 1,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton13), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton13_toggled),
                      NULL);

  radiobutton14 = gtk_radio_button_new_with_label (table3_group, "every 20%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton14));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton14", radiobutton14);
  gtk_widget_show (radiobutton14);
  gtk_table_attach (GTK_TABLE (table3), radiobutton14, 1, 2, 1, 2,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton14), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton14_toggled),
                      NULL);

  radiobutton15 = gtk_radio_button_new_with_label (table3_group, "every 25%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton15));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton15", radiobutton15);
  gtk_widget_show (radiobutton15);
  gtk_table_attach (GTK_TABLE (table3), radiobutton15, 1, 2, 2, 3,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton15), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton15_toggled),
                      NULL);

  radiobutton16 = gtk_radio_button_new_with_label (table3_group, "every 50%");
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton16));
  gtk_object_set_data (GTK_OBJECT (window3), "radiobutton16", radiobutton16);
  gtk_widget_show (radiobutton16);
  gtk_table_attach (GTK_TABLE (table3), radiobutton16, 1, 2, 3, 4,
                    (GtkAttachOptions) GTK_EXPAND | GTK_FILL, (GtkAttachOptions) GTK_EXPAND | GTK_FILL, 0, 0);
  gtk_signal_connect (GTK_OBJECT (radiobutton16), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton16_toggled),
                      NULL);

  hbox13 = gtk_hbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (window3), "hbox13", hbox13);
  gtk_widget_show (hbox13);
  gtk_box_pack_start (GTK_BOX (vbox12), hbox13, TRUE, TRUE, 0);

  button5 = gtk_button_new_with_label ("OK");
  gtk_object_set_data (GTK_OBJECT (window3), "button5", button5);
  gtk_widget_show (button5);
  gtk_box_pack_start (GTK_BOX (hbox13), button5, TRUE, FALSE, 0);
  gtk_signal_connect (GTK_OBJECT (button5), "clicked",
                      GTK_SIGNAL_FUNC (on_button5_clicked),
                      NULL);


  /* ץѥƥɥɽΥХåλ */
  propdata [0] = entry12;
  propdata [1] = entry13;
  propdata [2] = entry14;
  propdata [3] = radiobutton1;
  propdata [4] = radiobutton2;
  propdata [5] = radiobutton3;
  propdata [6] = radiobutton4;
  propdata [7] = radiobutton5;
  propdata [8] = radiobutton6;
  propdata [9] = radiobutton7;
  propdata [10] = radiobutton8;
  propdata [11] = radiobutton9;
  propdata [12] = radiobutton10;
  propdata [13] = radiobutton11;
  propdata [14] = radiobutton12;
  propdata [15] = radiobutton13;
  propdata [16] = radiobutton14;
  propdata [17] = radiobutton15;
  propdata [18] = radiobutton16;

  gtk_signal_connect(GTK_OBJECT(window3), "show",
		     GTK_SIGNAL_FUNC(on_window3_show), propdata);

  return window3;
}

GtkWidget*
create_dialog1 ()
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *label30;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox14;
  GtkWidget *button6;
  GtkWidget *button7;

  dialog1 = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog1", dialog1);
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (dialog1), "Really exit?");
  gtk_window_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (dialog1), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1);
  gtk_widget_show (dialog_vbox1);

  label30 = gtk_label_new ("Really exit?");
  gtk_object_set_data (GTK_OBJECT (dialog1), "label30", label30);
  gtk_widget_show (label30);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label30, FALSE, FALSE, 5);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1);
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox14 = gtk_hbox_new (TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (dialog1), "hbox14", hbox14);
  gtk_widget_show (hbox14);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox14, TRUE, TRUE, 0);

  button6 = gtk_button_new_with_label ("Exit");
  gtk_object_set_data (GTK_OBJECT (dialog1), "button6", button6);
  gtk_widget_show (button6);
  gtk_box_pack_start (GTK_BOX (hbox14), button6, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button6), "clicked",
                      GTK_SIGNAL_FUNC (on_button6_clicked),
                      NULL);

  button7 = gtk_button_new_with_label ("Cancel");
  gtk_object_set_data (GTK_OBJECT (dialog1), "button7", button7);
  gtk_widget_show (button7);
  gtk_box_pack_start (GTK_BOX (hbox14), button7, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button7), "clicked",
                      GTK_SIGNAL_FUNC (on_button7_clicked),
                      NULL);

  return dialog1;
}

GtkWidget*
create_dialog2 ()
{
  GtkWidget *dialog2;
  GtkWidget *dialog_vbox2;
  GtkWidget *vbox13;
  GtkWidget *label31;
  GtkWidget *entry15;
  GtkWidget *dialog_action_area2;
  GtkWidget *hbox15;
  GtkWidget *button8;
  GtkWidget *button9;

  dialog2 = gtk_dialog_new ();
  gtk_object_set_data (GTK_OBJECT (dialog2), "dialog2", dialog2);
  gtk_container_border_width (GTK_CONTAINER (dialog2), 6);
  GTK_WINDOW (dialog2)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (dialog2), "Print");
  gtk_window_position (GTK_WINDOW (dialog2), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (dialog2), TRUE, TRUE, FALSE);

  dialog_vbox2 = GTK_DIALOG (dialog2)->vbox;
  gtk_object_set_data (GTK_OBJECT (dialog2), "dialog_vbox2", dialog_vbox2);
  gtk_widget_show (dialog_vbox2);

  vbox13 = gtk_vbox_new (FALSE, 0);
  gtk_object_set_data (GTK_OBJECT (dialog2), "vbox13", vbox13);
  gtk_widget_show (vbox13);
  gtk_box_pack_start (GTK_BOX (dialog_vbox2), vbox13, TRUE, TRUE, 0);

  label31 = gtk_label_new ("Input print command:");
  gtk_object_set_data (GTK_OBJECT (dialog2), "label31", label31);
  gtk_widget_show (label31);
  gtk_box_pack_start (GTK_BOX (vbox13), label31, TRUE, TRUE, 0);

  entry15 = gtk_entry_new ();
  gtk_object_set_data (GTK_OBJECT (dialog2), "entry15", entry15);
  gtk_widget_show (entry15);
  gtk_box_pack_start (GTK_BOX (vbox13), entry15, TRUE, TRUE, 0);
  gtk_entry_set_text (GTK_ENTRY (entry15), "lpr -Plp");

  dialog_action_area2 = GTK_DIALOG (dialog2)->action_area;
  gtk_object_set_data (GTK_OBJECT (dialog2), "dialog_action_area2", dialog_action_area2);
  gtk_widget_show (dialog_action_area2);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area2), 10);

  hbox15 = gtk_hbox_new (TRUE, 0);
  gtk_object_set_data (GTK_OBJECT (dialog2), "hbox15", hbox15);
  gtk_widget_show (hbox15);
  gtk_box_pack_start (GTK_BOX (dialog_action_area2), hbox15, TRUE, TRUE, 0);

  button8 = gtk_button_new_with_label ("Print");
  gtk_object_set_data (GTK_OBJECT (dialog2), "button8", button8);
  gtk_widget_show (button8);
  gtk_box_pack_start (GTK_BOX (hbox15), button8, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button8), "clicked",
                      GTK_SIGNAL_FUNC (on_button8_clicked),
                      entry15);

  button9 = gtk_button_new_with_label ("Cancel");
  gtk_object_set_data (GTK_OBJECT (dialog2), "button9", button9);
  gtk_widget_show (button9);
  gtk_box_pack_start (GTK_BOX (hbox15), button9, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button9), "clicked",
                      GTK_SIGNAL_FUNC (on_button9_clicked),
                      NULL);

  return dialog2;
}

GtkWidget*
create_dialog3 (gchar *message, gchar*title_message)
{
  GtkWidget *dialog1;
  GtkWidget *dialog_vbox1;
  GtkWidget *label30;
  GtkWidget *dialog_action_area1;
  GtkWidget *hbox14;
  GtkWidget *button10;
  GtkWidget *button11;

  dialog1 = gtk_dialog_new ();
  GTK_WINDOW (dialog1)->type = GTK_WINDOW_DIALOG;
  gtk_window_set_title (GTK_WINDOW (dialog1), title_message);
  gtk_window_position (GTK_WINDOW (dialog1), GTK_WIN_POS_CENTER);
  gtk_window_set_policy (GTK_WINDOW (dialog1), TRUE, TRUE, FALSE);

  dialog_vbox1 = GTK_DIALOG (dialog1)->vbox;
  gtk_widget_show (dialog_vbox1);

  label30 = gtk_label_new (message);
  gtk_widget_show (label30);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), label30, FALSE, FALSE, 5);

  dialog_action_area1 = GTK_DIALOG (dialog1)->action_area;
  gtk_widget_show (dialog_action_area1);
  gtk_container_border_width (GTK_CONTAINER (dialog_action_area1), 10);

  hbox14 = gtk_hbox_new (TRUE, 0);
  gtk_widget_show (hbox14);
  gtk_box_pack_start (GTK_BOX (dialog_action_area1), hbox14, TRUE, TRUE, 0);

  button10 = gtk_button_new_with_label ("Yes");
  gtk_widget_show (button10);
  gtk_box_pack_start (GTK_BOX (hbox14), button10, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button10), "clicked",
                      GTK_SIGNAL_FUNC (on_button10_clicked),
                      NULL);

  button11 = gtk_button_new_with_label ("No");
  gtk_widget_show (button11);
  gtk_box_pack_start (GTK_BOX (hbox14), button11, TRUE, TRUE, 0);
  gtk_signal_connect (GTK_OBJECT (button11), "clicked",
                      GTK_SIGNAL_FUNC (on_button11_clicked),
                      NULL);

  return dialog1;
}
